"""############################################################################ 
2019/4/20 Start
source catalogue extraction from images
""" ############################################################################
from __future__ import print_function
from builtins import input
import os
import sys
import time
from astropy.io import fits
import numpy as np
import pylab as pl
import matplotlib.pyplot as plt
from imutils import contours
from skimage import measure
import imutils
import cv2

def zscale(image, nsamples=1000, contrast=0.25, max_reject=0.5, min_npixels=5,
           krej=2.5, max_iterations=5):
 
    # Sample the image
    image = np.asarray(image)
    image = image[np.isfinite(image)]
    stride = int(max(1.0, image.size / nsamples))
    samples = image[::stride][:nsamples]
    samples.sort()

    npix = len(samples)
    zmin = samples[0]
    zmax = samples[-1]

    # Fit a line to the sorted array of samples
    minpix = max(min_npixels, int(npix * max_reject))
    x = np.arange(npix)
    ngoodpix = npix
    last_ngoodpix = npix + 1

    # Bad pixels mask used in k-sigma clipping
    badpix = np.zeros(npix, dtype=bool)

    # Kernel used to dilate the bad pixels mask
    ngrow = max(1, int(npix * 0.01))
    kernel = np.ones(ngrow, dtype=bool)

    for niter in range(max_iterations):
        if ngoodpix >= last_ngoodpix or ngoodpix < minpix:
            break

        fit = np.polyfit(x, samples, deg=1, w=(~badpix).astype(int))
        fitted = np.poly1d(fit)(x)

        # Subtract fitted line from the data array
        flat = samples - fitted

        # Compute the k-sigma rejection threshold
        threshold = krej * flat[~badpix].std()

        # Detect and reject pixels further than k*sigma from the fitted line
        badpix[(flat < - threshold) | (flat > threshold)] = True

        # Convolve with a kernel of length ngrow
        badpix = np.convolve(badpix, kernel, mode='same')

        last_ngoodpix = ngoodpix
        ngoodpix = np.sum(~badpix)

    slope, intercept = fit

    if ngoodpix >= minpix:
        if contrast > 0:
            slope = slope / contrast
        center_pixel = (npix - 1) // 2
        median = np.median(samples)
        zmin = max(zmin, median - (center_pixel - 1) * slope)
        zmax = min(zmax, median + (npix - center_pixel) * slope)
    return zmin, zmax

def imshow(data,_fig,_size,vmin,vmax):

    _xdim,_ydim = data.shape
    fig = plt.figure(_fig,figsize=(_size*_ydim/_xdim,_size))
    plt.imshow(data,cmap='gray',aspect='auto',vmin=vmin, vmax=vmax, origin='lower')
    plt.gca().invert_yaxis()
    plt.axis('off')
    plt.tight_layout()
    return fig

def sfd(img, SNR = 10, sizethreshold = 20,verbose = False, figsize = 8):
    '''functions for source finder
    '''

    if verbose:pl.ion()
    with fits.open(img) as hdul:
        if verbose:  hdul.info()
        data = hdul[0].data   
        vmin,vmax = zscale(data)     

        if verbose:
            print('%.2f-%.2f'%(vmin,vmax))
            fig1=imshow(data,1,figsize,vmin,vmax)
            input('1')

        data1 = cv2.threshold(data, vmin+SNR*(vmax-vmin), vmax, cv2.THRESH_BINARY)[1]
        data1 = cv2.erode(data1, None, iterations=2)
        data1 = cv2.dilate(data1, None, iterations=4)

        '''
        if verbose:
            fig1=imshow(data1,2,figsize,vmin,vmax)
            input('1')
            '''

        labels = measure.label(data1, neighbors=8, background=0)        
        mask = np.zeros(data1.shape, dtype="uint8")

        for label in np.unique(labels):
            if label == 0:continue        

            _ii = int(label/len(np.unique(labels))*100)
            sys.stdout.write(('='*_ii)+(''*(100-_ii))+("\r [ %d"%_ii+"% ] "))
            sys.stdout.flush()

            labelMask = np.zeros(data1.shape, dtype="uint8")
            labelMask[labels == label] = 255
            numPixels = cv2.countNonZero(labelMask)
        
            if numPixels > sizethreshold:mask = cv2.add(mask, labelMask)

        '''
        if verbose:
            imshow(mask,2,figsize,0,1)
            input('2')
            '''

        cnts = cv2.findContours(mask.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
        cnts = imutils.grab_contours(cnts)
        cnts = contours.sort_contours(cnts)[0]

        for (i, c) in enumerate(cnts):
            print(len(c))

            (x, y, w, h) = cv2.boundingRect(c)
            print(x, y, w, h)
        
            ((X_IMAGE, Y_IMAGE), SIZE) = cv2.minEnclosingCircle(c)
            print(X_IMAGE, Y_IMAGE, SIZE)

            ellipse = cv2.fitEllipse(c)
            print(ellipse)

            epsilon = 0.1*cv2.arcLength(c,True)
            print(epsilon)
            input('...')

            if verbose:
                print(X_IMAGE, Y_IMAGE, SIZE)
                circle=plt.Circle((X_IMAGE, Y_IMAGE),radius=SIZE, color='r',fill=False)
                fig1.gca().add_artist(circle)
        input('...')
